/*
	Example embedding a tcl interpreter 
	for GUI enabling a Cobol program.

	Recife, Brazil, 2001 -- Rildo Pragana
*/

#include <tcl.h>
#include <tk.h>
#include <stdio.h>
#include <stdlib.h>

struct coboldata {
	int cmd;
	int event;
	char gui_name[40];
	int attr_type;
	char attr_value[80];
	int widget_sel;
	int cobsize;
	char cobolvars[1];
} *coboldata;

#include "tcguirt.c"

Tcl_Interp *interp=NULL;
int error_code;
Tk_Window mainwin;

void tcgui_do_exit() {
	char *einf;
	einf = Tcl_GetVar(interp,"errorInfo",TCL_GLOBAL_ONLY);
	fprintf(stderr,"*** TCGUI ERROR from Tcl interpreter:\n%s\n",einf);
	exit(1);
}

int
Tcl_AppInit(Tcl_Interp *interp) {
    if (Tcl_Init(interp) == TCL_ERROR) {
	return TCL_ERROR;
    }
    if (Tk_Init(interp) == TCL_ERROR) {
	return TCL_ERROR;
    }
    Tcl_StaticPackage(interp, "Tk", Tk_Init, Tk_SafeInit);
    Tcl_SetVar(interp, "tcl_rcFileName", "~/.wishrc", TCL_GLOBAL_ONLY);
    return TCL_OK;
}

int
getcobvar(ClientData *clientData, Tcl_Interp *interp, 
	int argc, char *argv[] ) {
	int offset,size;
	char *s;
	if (argc!=4) {
		Tcl_SetResult(interp, "wrong number of arguments",
			TCL_STATIC);
		return TCL_ERROR;
	}
	Tcl_GetInt(interp,argv[2],&offset);
	Tcl_GetInt(interp,argv[3],&size);
	Tcl_SetVar2Ex(interp,argv[1],NULL,
		Tcl_NewStringObj(coboldata->cobolvars+offset,size),0);	
	return TCL_OK;
}

int
setcobvar(ClientData *clientData, Tcl_Interp *interp, 
	int argc, char *argv[] ) {
	int offset,size;
	struct coboldata *data;
	if (argc!=5) {
		Tcl_SetResult(interp, "wrong number of arguments",
			TCL_STATIC);
		return TCL_ERROR;
	}
	Tcl_GetInt(interp,argv[2],&offset);
	Tcl_GetInt(interp,argv[3],&size);
	Tcl_GetInt(interp,argv[4],(int *)&data);

	memmove(data->cobolvars+offset,argv[1],size);
	return TCL_OK;
}

int initTcl () {
	int argc=0;
	char *argv[]= { "tinycobol", ""};
	if (interp != NULL) {
		return;
	}
	interp = Tcl_CreateInterp();
	Tk_MainWindow(interp);
	Tcl_AppInit(interp);
	Tcl_CreateCommand(interp, "getcobvar", 
		(Tcl_CmdProc *)getcobvar, NULL, NULL);
	Tcl_CreateCommand(interp, "setcobvar", 
		(Tcl_CmdProc *)setcobvar, NULL, NULL);
	error_code = Tcl_Eval(interp,tcguirt);
	if (error_code != TCL_OK)
		tcgui_do_exit();
}

int endTcl () {
	Tcl_DeleteInterp(interp);
}

void trimright ( char *s ) {
	char *s1=s;
	s1 += strlen(s)-1;
	while (*s1 == ' ') s1--;
	*(s1+1)=0;
}

int TCGUI(struct coboldata *data) {
	char *s,*compscript;
	char buf[81];
	int *iptr;
	coboldata = data;
	initTcl();
	Tcl_SetVar2Ex(interp,"tc","cmd",Tcl_NewIntObj(data->cmd),
		TCL_GLOBAL_ONLY);
	Tcl_SetVar2Ex(interp,"tc","event",Tcl_NewIntObj(data->event),
		TCL_GLOBAL_ONLY);
	data->event = 0;
	strncpy(buf,data->gui_name,40);
	buf[40]=0;
	trimright(buf);
    Tcl_SetVar2(interp, "tc", "gui_name",buf,TCL_GLOBAL_ONLY);
	strncpy(buf,data->attr_value,80);
	buf[80]=0;
	trimright(buf);
    Tcl_SetVar2(interp, "tc", "attr_value",buf,TCL_GLOBAL_ONLY);
	Tcl_SetVar2Ex(interp,"tc","attr_type",Tcl_NewIntObj(data->attr_type),
		TCL_GLOBAL_ONLY);
	Tcl_SetVar2Ex(interp,"tc","widget_sel",Tcl_NewIntObj(data->widget_sel),
		TCL_GLOBAL_ONLY);
	Tcl_SetVar2Ex(interp,"tc","cobsize",Tcl_NewIntObj(data->cobsize),
		TCL_GLOBAL_ONLY);
	/* setup address of this data structure too */
	Tcl_SetVar2Ex(interp,"tc","data_ptr",Tcl_NewIntObj((int)data),
		TCL_GLOBAL_ONLY);

	iptr = (int *)&(data->cobolvars[data->cobsize]);
	if (*iptr == 0) {
		(*iptr)++;
		s = compscript = (char *)(iptr+1);
		while (*s != ' ') s++;
		*s = 0;
		//Tcl_EvalFile(interp,"test.tcl");
		Tcl_SetVar(interp,"encoded_source",compscript,TCL_GLOBAL_ONLY);
		error_code = Tcl_Eval(interp,"exec_encoded");
		if (error_code != TCL_OK)
			tcgui_do_exit();
	}
	error_code = Tcl_Eval(interp,"get_cobvars");
	if (error_code != TCL_OK)
		tcgui_do_exit();
	
	s = Tcl_GetVar2(interp,"tc","data_ptr",
		TCL_GLOBAL_ONLY);
	error_code = Tcl_GetInt(interp,s,(int *)&data);
	if (error_code != TCL_OK)
		tcgui_do_exit();
	s = Tcl_GetVar2(interp,"tc","event",TCL_GLOBAL_ONLY);
	error_code = Tcl_GetInt(interp,s,&data->event);
	if (error_code != TCL_OK)
		tcgui_do_exit();
	s = Tcl_GetVar2(interp,"tc","widget_sel",TCL_GLOBAL_ONLY);
	error_code = Tcl_GetInt(interp,s,&data->widget_sel);
	if (error_code != TCL_OK)
		tcgui_do_exit();
}

